// HTextureStore.cpp: implementation of the HTextureStore class.
//
//////////////////////////////////////////////////////////////////////

#include "ph.h"
#include "HTextureStore.h"

HTextureStore htexture;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

HTextureStore::HTextureStore()
{
	// nulovani dat
	m_textures = 0;
	for (int i = 0; i < MAX_TEXTURES; i++) m_texture[i] = NULL;

	// defaultove parametry
	m_mipmaps    = false;
	m_min_filter = GL_LINEAR;
	m_mag_filter = GL_LINEAR;
}

HTextureStore::~HTextureStore()
{
	// smazani seznamu textur
	for (int i = 0; i < MAX_TEXTURES; i++) DestroyTexture(i);
}

int HTextureStore::GetTexture(const char *file)
{
	return GetTexture(file, m_min_filter, m_mag_filter, m_mipmaps);
}

int HTextureStore::GetTexture(const char* file, int min_filter, int mag_filter, bool mipmaps)
{
	// hledani jiz jednou vytvorene textury

	// v pripade neuspechu vytvoreni nove
	return CreateTexture(file, min_filter, mag_filter, mipmaps);
}

int HTextureStore::CreateTexture(const char* file, int min_filter, int mag_filter, bool mipmaps)
{
	// test na volne misto
	if (m_textures >= MAX_TEXTURES) {
		fprintf(stderr, "file: %s, line: %d MAX_TEXTURES too small - please use larger number.\n", __FILE__, __LINE__);
		return -1;
	}

	// nacteni obrazku
	SDL_Surface *surface;
	surface = LoadBitmap(file);
	if (surface == NULL) return -1;

	// vytvoreni objektu textury
	st_texture *t = new st_texture;
	if (t == NULL) return -1;

	// vygenerovani textury
	glGenTextures(1, &t->gl_texture);

	// nastaveni textury
	glBindTexture(GL_TEXTURE_2D, t->gl_texture);

	// predani parametru
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, mag_filter);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, min_filter);

	// pridani barevne mapy (obrazku) - pripadne generovani mip map
	if (mipmaps) 
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, surface->w, surface->h, GL_RGB, GL_UNSIGNED_BYTE, surface->pixels);
	else
		glTexImage2D(GL_TEXTURE_2D, 0, 3, surface->w, surface->h, 0, GL_RGB, GL_UNSIGNED_BYTE, surface->pixels);

	// pridani textury do seznamu
	int i;
	for (i = 0; i < MAX_TEXTURES; i++) 
		if (m_texture[i] == NULL) break;

	strcpy(t->file, file);
	t->used = 1;
	m_texture[i] = t;
	m_textures++;

	// uklid - smazani SDL_Surface
	delete surface;

	// konec 
	return i;
}

void HTextureStore::SetTexture(int ID)
{
	glColor3f(1.0, 1.0, 1.0);
	if (ID == -1) return;

	if (m_texture[ID] != NULL) {
 		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, m_texture[ID]->gl_texture);
	}
}

void HTextureStore::SetNullTexture()
{
	glBindTexture(GL_TEXTURE_2D, NULL);  // nastaveni NULL textury
	glDisable(GL_TEXTURE_2D);			    // zakaz texturovani
	glColor3ub(255, 255, 255);           // Reset the color to normal again
}

/////////////////////////////////////////////////////////////////////////////////////////

SDL_Surface* HTextureStore::LoadBitmap(const char *filename)
{
   Uint8 *rowhi, *rowlo;
   Uint8 *tmpbuf, tmpch;
   SDL_Surface *image;
   int i, j;

	// nacteni obrazkovych dat
	image = IMG_Load(filename);
   if ( image == NULL ) {
		fprintf(stderr, "Unable to load %s: %s\n", filename, SDL_GetError());
      return(NULL);
   }

    // GL surfaces jsou vzhuru nohama
   tmpbuf = (Uint8 *)malloc(image->pitch);

   if ( tmpbuf == NULL ) {
       fprintf(stderr, "Out of memory\n");
       return NULL;
   }

   rowhi = (Uint8 *)image->pixels;
   rowlo = rowhi + (image->h * image->pitch) - image->pitch;
   for ( i=0; i<image->h/2; ++i ) {
		// prevraceni BGR na RGB
		if (image->format->Bshift == 0)
			for ( j=0; j<image->w; ++j ) {
            tmpch = rowhi[j*3];
            rowhi[j*3] = rowhi[j*3+2];
            rowhi[j*3+2] = tmpch;
            tmpch = rowlo[j*3];
            rowlo[j*3] = rowlo[j*3+2];
            rowlo[j*3+2] = tmpch;
			}

      memcpy(tmpbuf, rowhi, image->pitch);
      memcpy(rowhi, rowlo, image->pitch);
      memcpy(rowlo, tmpbuf, image->pitch);
      rowhi += image->pitch;
      rowlo -= image->pitch;
   }

   free(tmpbuf);

   return image;
}

/////////////////////////////////////////////////////////////////////////////////////////

void HTextureStore::RemoveTexture(int ID)
{
	if (ID == -1 || m_texture[ID] == NULL) return;

	// smazeme jedno pouziti a zkusime, zda na texturu jeste nekdo odkazuje 
 	if (--m_texture[ID]->used <= 0) {
      // fyzicke zruseni textury

	}
}

void HTextureStore::DestroyTexture(int ID)
{
	if (m_texture[ID] == NULL) return;

 	glDeleteTextures(1, &m_texture[ID]->gl_texture);
	
	delete m_texture[ID];
	m_texture[ID] = NULL;
}

// nastavi qvalitu nacitanych textur
void HTextureStore::SetTextureGeneratingQuality(int quality)
{
	switch (quality) {
	case 0 :
		m_mipmaps    = false;
		m_min_filter = GL_NEAREST;
		m_mag_filter = GL_NEAREST;
		break;
	case 1 :
		m_mipmaps    = false;
		m_min_filter = GL_LINEAR;
		m_mag_filter = GL_LINEAR;
		break;
	case 2 :
		m_mipmaps    = true;
		m_min_filter = GL_LINEAR_MIPMAP_NEAREST;
		m_mag_filter = GL_LINEAR;
		break;
	}
}
